/*
  Created By Larry Apolonio
  March 10, 2018
  
  This sketch is for the LinuxFest 2018 Transmitter

  March 18, 2018
  RC1 version of demo
  Cleanup and Compile using Updated Arduino

  March 24, 2018
  Change PIR Code To generic one
  Added codes for 433Mhz Keypad

  March 27, 2018
  Added Pulse Width to transmit
  Added descriptiive OLED messages

  April 26, 2018
  Changed Wifi Password  
*/
#include <Wire.h>
#include <ESP8266WiFi.h>
#include <MQTTClient.h>
#include <RCSwitch.h>
#include "SSD1306.h"
#include "SSD1306Ui.h"
#include "font1.h"
#include "font2.h"


///////////////////////////////////////////////////////////////
// Global Definitions
const char* SKETCH = "RF433_TX_Linuxfest2018_Wemos";
const char* VERSION = "20180426";
const int messagedelay=2500;
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// ESP8266 Pins Definitions
const byte TXPIN = D8;
const byte RXPIN = D7;
const byte LEDPIN = 16;
const byte SDA1=D5;
const byte SCL1=D6;
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// Wifi Definitions Here
const char* ssid = "makernet";
const char* password = "ABcd1234"; 
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// MQTT Definitions Here
const char* mqttserver = "openhab.makernet.local";           // server or URL of MQTT broker
const char* mqttusername = "coco";                           // username for MQTT broker
const char* mqttpassword = "RememberMe2017";                 // password for MQTT broker 
String clientName = "RF433MHZ-TRANSMIT";                     // hostname used to talk to MQTT broker
 // subscribe to this topic; anything sent here will be passed into the messageReceived function
const char* subscribeTopic = "linuxfest/present/TXrh433mhz";
const char* publishTopic = "linuxfest/present/RXrh433mhz";
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// Declare Objects Here
WiFiClient wifiClient;
MQTTClient mqttClient;
RCSwitch txSwitch = RCSwitch();
SSD1306   display(0x3c, SDA1, SCL1);
SSD1306Ui ui     ( &display );
///////////////////////////////////////////////////////////////

void setup() {
  Serial.begin(115200);
  Serial.println();
///////////////////////////////////////////////////////////////
// 433Mhz Initialization
//  rxSwitch.enableReceive(RXPIN);
  txSwitch.enableTransmit(TXPIN);
  // Optional set pulse length.
  txSwitch.setPulseLength(162);
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// OLED Initialization Display 
  ui.init();
  display.flipScreenVertically();
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.clear();
  display.setFont(Serif_bold_12);
  display.drawStringMaxWidth(0, 0, 128, "433Mhz Receiver");
  display.setFont(SansSerif_plain_12);
  display.drawStringMaxWidth(0, 16, 128, SKETCH);
  display.drawStringMaxWidth(0, 32, 128, VERSION);
  display.display();
  delay(messagedelay);
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// Wifi Initialization
  Serial.print("Connecting to ");
  Serial.println(ssid);  
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  display.clear();  
  display.setFont(Serif_bold_12);
  display.drawStringMaxWidth(0, 0, 128, "433Mhz Transmit");
  display.setFont(SansSerif_plain_12);
  display.drawStringMaxWidth(0, 20, 128, "Connected");
  display.drawStringMaxWidth(0, 30, 128, WiFi.localIP().toString());
  display.display();
  delay(messagedelay);
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// MQTT Initialization
  mqttClient.begin(mqttserver,wifiClient);
  delay(1000);
  
  Serial.print("Connecting to ");
  Serial.print(mqttserver);
  Serial.print(" as ");
  Serial.println(clientName);
  
  if (mqttClient.connect((char*) clientName.c_str(),mqttusername,mqttpassword)) {
    Serial.println("Connected to MQTT broker");
    Serial.print("Subscribed to: ");
    Serial.println(subscribeTopic);
    mqttClient.subscribe(subscribeTopic);

  }
  else {
    Serial.println("MQTT connect failed");
    Serial.println("Will reset and try again...");
    abort();
  }
  display.drawStringMaxWidth(0, 40, 128, "MQTT Connected");
  display.drawStringMaxWidth(0, 50, 128, "Ready to begin");
  display.display();
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// Final Message it is ready
  display.clear();
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.setFont(Serif_bold_12);
  display.drawStringMaxWidth(0, 0, 128, "433Mhz Transmit");
  display.setFont(SansSerif_plain_12);
  display.drawStringMaxWidth(0, 16, 128, "Arduino, ESP8266");
  display.drawStringMaxWidth(0, 32, 128, "and 433Mhz Devices");
  display.drawStringMaxWidth(0, 48, 128, "By Larry Apolonio");
  display.display();
  delay(messagedelay);
}
///////////////////////////////////////////////////////////////

void loop() {
  // put your main code here, to run repeatedly:
  //rh433loop();
  // Required to process incoming mqtt messages
  mqttClient.loop();
}

void messageReceived(String topic, String payload, char * bytes, unsigned int length) {
  unsigned long tx_code;
  tx_code = payload.toInt();
  Serial.print("incoming: ");
  Serial.print(topic);
  Serial.print(" - ");
  Serial.print(payload);
  Serial.println();
  display.clear();
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.setFont(Serif_bold_12);
  display.drawStringMaxWidth(0, 0, 128, "433Mhz Transmit");
  display.setFont(SansSerif_plain_12);
  display.drawStringMaxWidth(0, 16, 128, payload);
  display.display();
  sendcode(tx_code);
}
