/*
  Created By Larry Apolonio
  March 21, 2018
  
  This sketch is a simple MQTT to 433Mhz Transmitter

  April 26, 2018
  Changed Wifi Password

*/
#include <Wire.h>
#include <ESP8266WiFi.h>
#include <MQTTClient.h>
#include <RCSwitch.h>


///////////////////////////////////////////////////////////////
// ESP8266 Pins Definitions
const byte TXPIN = D8;
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// Wifi Definitions Here
const char* ssid = "makernet";
const char* password = "ABcd1234"; 
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// MQTT Definitions Here
const char* mqttserver = "openhab.makernet.local";           // server or URL of MQTT broker
const char* mqttusername = "coco";                           // username for MQTT broker
const char* mqttpassword = "RememberMe2017";                 // password for MQTT broker 
String clientName = "RF433MHZ-TRANSMIT";                     // hostname used to talk to MQTT broker
 // subscribe to this topic; anything sent here will be passed into the messageReceived function
const char* subscribeTopic = "linuxfest/present/TXrh433mhz";
const char* publishTopic = "linuxfest/present/RXrh433mhz";
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// Declare Objects Here
WiFiClient wifiClient;
MQTTClient mqttClient;
RCSwitch txSwitch = RCSwitch();
///////////////////////////////////////////////////////////////

void setup() {
  Serial.begin(115200);
  Serial.println();
///////////////////////////////////////////////////////////////
// 433Mhz Initialization
//  rxSwitch.enableReceive(RXPIN);
  txSwitch.enableTransmit(TXPIN);
  // Optional set pulse length.
  txSwitch.setPulseLength(162);
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// Wifi Initialization
  Serial.print("Connecting to ");
  Serial.println(ssid);  
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
///////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////
// MQTT Initialization
  mqttClient.begin(mqttserver,wifiClient);
  delay(1000);
  
  Serial.print("Connecting to ");
  Serial.print(mqttserver);
  Serial.print(" as ");
  Serial.println(clientName);
  
  if (mqttClient.connect((char*) clientName.c_str(),mqttusername,mqttpassword)) {
    Serial.println("Connected to MQTT broker");
    Serial.print("Subscribed to: ");
    Serial.println(subscribeTopic);
    mqttClient.subscribe(subscribeTopic);

  }
  else {
    Serial.println("MQTT connect failed");
    Serial.println("Will reset and try again...");
    abort();
  }
///////////////////////////////////////////////////////////////
}

void loop() {
  mqttClient.loop();
  delay(10);
}

void messageReceived(String topic, String payload, char * bytes, unsigned int length) {
  unsigned long tx_code;
  tx_code = payload.toInt();
  Serial.print("incoming: ");
  Serial.print(topic);
  Serial.print(" - ");
  Serial.print(payload);
  Serial.println();
  txSwitch.send(tx_code, 24);
}
